@@ -184,6 +184,10 @@ protocol_choosers_add_cm (EmpathyProtocolChooser *chooser,
         /* Haze's SIP implementation is pretty useless (bgo #629736) */
         continue;
 
+      if (!tp_strdiff (cm->name, "butterfly"))
+        /* Butterfly isn't supported any more */
+        continue;
+
       if (tp_strdiff (cm->name, "haze") && !tp_strdiff (saved_cm_name, "haze"))
         {
           GtkTreeIter titer;
@@ -365,7 +369,7 @@ protocol_chooser_finalize (GObject *object)
 
   if (priv->protocols)
     {
-      g_hash_table_destroy (priv->protocols);
+      g_hash_table_unref (priv->protocols);
       priv->protocols = NULL;
     }
 
