@@ -47,6 +47,7 @@
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-request-util.h>
+#include <libempathy/empathy-individual-manager.h>
 
 #include <libempathy-gtk/empathy-images.h>
 #include <libempathy-gtk/empathy-contact-dialogs.h>
@@ -129,12 +130,14 @@ static const guint tab_accel_keys[] = {
 
 typedef enum {
 	DND_DRAG_TYPE_CONTACT_ID,
+	DND_DRAG_TYPE_INDIVIDUAL_ID,
 	DND_DRAG_TYPE_URI_LIST,
 	DND_DRAG_TYPE_TAB
 } DndDragType;
 
 static const GtkTargetEntry drag_types_dest[] = {
 	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
+	{ "text/x-individual-id", 0, DND_DRAG_TYPE_INDIVIDUAL_ID },
 	{ "GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, DND_DRAG_TYPE_TAB },
 	{ "text/uri-list", 0, DND_DRAG_TYPE_URI_LIST },
 	{ "text/path-list", 0, DND_DRAG_TYPE_URI_LIST },
@@ -142,6 +145,7 @@ static const GtkTargetEntry drag_types_dest[] = {
 
 static const GtkTargetEntry drag_types_dest_contact[] = {
 	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
+	{ "text/x-individual-id", 0, DND_DRAG_TYPE_INDIVIDUAL_ID },
 };
 
 static const GtkTargetEntry drag_types_dest_file[] = {
@@ -920,6 +924,19 @@ chat_window_update_chat_tab_full (EmpathyChat *chat,
 		append_markup_printf (tooltip, "\n%s", _("Typing a message."));
 	}
 
+	if (remote_contact != NULL) {
+		const gchar * const *types;
+
+		types = empathy_contact_get_client_types (remote_contact);
+		if (types != NULL && !tp_strdiff (types[0], "phone")) {
+			/* I'm on a phone ! */
+			gchar *tmp = name;
+
+			name = g_strdup_printf ("☎ %s", name);
+			g_free (tmp);
+		}
+	}
+
 	markup = g_string_free (tooltip, FALSE);
 	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-tooltip-widget");
 	gtk_widget_set_tooltip_markup (widget, markup);
@@ -928,10 +945,19 @@ chat_window_update_chat_tab_full (EmpathyChat *chat,
 	g_free (markup);
 
 	/* Update tab and menu label */
+	if (empathy_chat_is_highlighted (chat)) {
+		markup = g_markup_printf_escaped (
+			"<span color=\"red\" weight=\"bold\">%s</span>",
+			name);
+	} else {
+		markup = g_markup_escape_text (name, -1);
+	}
+
 	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-label");
-	gtk_label_set_text (GTK_LABEL (widget), name);
+	gtk_label_set_markup (GTK_LABEL (widget), markup);
 	widget = g_object_get_data (G_OBJECT (chat), "chat-window-menu-label");
-	gtk_label_set_text (GTK_LABEL (widget), name);
+	gtk_label_set_markup (GTK_LABEL (widget), markup);
+	g_free (markup);
 
 	/* Update the window if it's the current chat */
 	if (priv->current_chat == chat) {
@@ -1560,30 +1586,6 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 	g_free (escaped);
 }
 
-static void
-chat_window_set_highlight_room_labels (EmpathyChat *chat)
-{
-	gchar *markup, *name;
-	GtkWidget *widget;
-
-	if (!empathy_chat_is_room (chat))
-		return;
-
-	name = empathy_chat_dup_name (chat);
-	markup = g_markup_printf_escaped (
-		"<span color=\"red\" weight=\"bold\">%s</span>",
-		name);
-
-	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-label");
-	gtk_label_set_markup (GTK_LABEL (widget), markup);
-
-	widget = g_object_get_data (G_OBJECT (chat), "chat-window-menu-label");
-	gtk_label_set_markup (GTK_LABEL (widget), markup);
-
-	g_free (name);
-	g_free (markup);
-}
-
 static gboolean
 empathy_chat_window_has_focus (EmpathyChatWindow *window)
 {
@@ -1670,8 +1672,6 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 	}
 
 	if (needs_urgency) {
-		chat_window_set_highlight_room_labels (chat);
-
 		if (!has_focus) {
 			chat_window_set_urgency_hint (window, TRUE);
 		}
@@ -1984,6 +1984,70 @@ chat_window_drag_motion (GtkWidget        *widget,
 	return FALSE;
 }
 
+static void
+drag_data_received_individual_id (EmpathyChatWindow *self,
+				  GtkWidget *widget,
+				  GdkDragContext *context,
+				  int x,
+				  int y,
+				  GtkSelectionData *selection,
+				  guint info,
+				  guint time_)
+{
+	const gchar *id;
+	EmpathyIndividualManager *manager = NULL;
+	FolksIndividual *individual;
+	EmpathyChatWindowPriv *priv = GET_PRIV (self);
+	EmpathyTpChat *chat;
+	TpContact *tp_contact;
+	TpConnection *conn;
+	EmpathyContact *contact;
+
+	id = (const gchar *) gtk_selection_data_get_data (selection);
+
+	DEBUG ("DND invididual %s", id);
+
+	if (priv->current_chat == NULL)
+		goto out;
+
+	chat = empathy_chat_get_tp_chat (priv->current_chat);
+	if (chat == NULL)
+		goto out;
+
+	if (!empathy_tp_chat_can_add_contact (chat)) {
+		DEBUG ("Can't invite contact to %s",
+				tp_proxy_get_object_path (chat));
+		goto out;
+	}
+
+	manager = empathy_individual_manager_dup_singleton ();
+
+	individual = empathy_individual_manager_lookup_member (manager, id);
+	if (individual == NULL) {
+		DEBUG ("Failed to find individual %s", id);
+		goto out;
+	}
+
+	conn = tp_channel_borrow_connection ((TpChannel *) chat);
+	tp_contact = empathy_get_tp_contact_for_individual (individual, conn);
+	if (tp_contact == NULL) {
+		DEBUG ("Can't find a TpContact on connection %s for %s",
+				tp_proxy_get_object_path (conn), id);
+		goto out;
+	}
+
+	DEBUG ("Inviting %s to join %s", tp_contact_get_identifier (tp_contact),
+			tp_channel_get_identifier ((TpChannel *) chat));
+
+	contact = empathy_contact_dup_from_tp_contact (tp_contact);
+	empathy_contact_list_add (EMPATHY_CONTACT_LIST (chat), contact, NULL);
+	g_object_unref (contact);
+
+out:
+	gtk_drag_finish (context, TRUE, FALSE, time_);
+	tp_clear_object (&manager);
+}
+
 static void
 chat_window_drag_data_received (GtkWidget        *widget,
 				GdkDragContext   *context,
@@ -2064,6 +2128,10 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		 */
 		gtk_drag_finish (context, TRUE, FALSE, time_);
 	}
+	else if (info == DND_DRAG_TYPE_INDIVIDUAL_ID) {
+		drag_data_received_individual_id (window, widget, context, x, y,
+				selection, info, time_);
+	}
 	else if (info == DND_DRAG_TYPE_URI_LIST) {
 		EmpathyChatWindowPriv *priv;
 		EmpathyContact *contact;
